/*-----------------------------------------------------------------------
Copyright 2010-2016 S. GRAILLAT, F. JEZEQUEL
Copyright 2010 S. WANG, Y. ZHU
Copyright 2019-2020 F. JEZEQUEL, J.-L. LAMOTTE, J.E. TIDSWELL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
// 
///////////////////////////////////////////////////
// Last Modification 
//
//
///////////////////////////////////////////////////
//
//contains the operators related to addition: +, ++, +=      
//      
//  /

inline mp_st<T> operator+() const & // l-value / copy
{
	return *this;
}

inline mp_st<T> operator+() && // r-value / move
{
	return std::move(*this);
}

mp_st<T> operator++()
{
	int initAccuracy = this->accuracy;
	if (_SAM_cancel_tag && (initAccuracy==DIGIT_NOT_COMPUTED)) {
		initAccuracy = this->nb_significant_digit();
	}

	accuracy=DIGIT_NOT_COMPUTED;
	if (SAM_RANDOM) mpfr_add_ui (x, x, 1, MPFR_RNDU);
	else mpfr_add_ui (x, x, 1, MPFR_RNDD);
	if (SAM_RANDOM){
		mpfr_add_ui (y, y, 1, MPFR_RNDU);
		mpfr_add_ui (z, z, 1, MPFR_RNDD);
	}
	else {
		mpfr_add_ui (y, y, 1, MPFR_RNDD);
		mpfr_add_ui (z, z, 1, MPFR_RNDU);
	}
	
	if (_SAM_cancel_tag){
		if (initAccuracy >=this->nb_significant_digit()+_SAM_cancel_value)
			instability(&_SAM_cancel_count);
	}
	return *this;
};

///////////////////////////////////////////////////////////////////////////////////////////


mp_st<T> operator++(int)
{
	mp_st initial=*this;
	
	accuracy=DIGIT_NOT_COMPUTED;
	if (SAM_RANDOM) mpfr_add_ui (x, x, 1, MPFR_RNDU);
	else mpfr_add_ui (x, x, 1, MPFR_RNDD);
	if (SAM_RANDOM){
		mpfr_add_ui (y, y, 1, MPFR_RNDU);
		mpfr_add_ui (z, z, 1, MPFR_RNDD);
	}
	else {
		mpfr_add_ui (y, y, 1, MPFR_RNDD);
		mpfr_add_ui (z, z, 1, MPFR_RNDU);
	}
	
	if (_SAM_cancel_tag){
	    if (initial.accuracy==DIGIT_NOT_COMPUTED) initial.nb_significant_digit();
	    if (initial.accuracy >=  this->nb_significant_digit()+_SAM_cancel_value)
		instability(&_SAM_cancel_count);
	}
	return initial;
}


///////////////////////////////////////////////////////////////////////////////////////////


template<typename arg> 
mp_st<T>& operator+=(const arg& a)
{
    int initAccuracy = this->accuracy;
	if (_SAM_cancel_tag && (initAccuracy==DIGIT_NOT_COMPUTED)) {
		initAccuracy = this->nb_significant_digit();
	}
	
	accuracy=DIGIT_NOT_COMPUTED;
	if (SAM_RANDOM) SAM::add_mpfr (x, x, a, MPFR_RNDU);
	else SAM::add_mpfr (x, x, a, MPFR_RNDD);
	if (SAM_RANDOM){
		SAM::add_mpfr (y, y, a, MPFR_RNDU);
		SAM::add_mpfr (z, z, a, MPFR_RNDD);
	}
	else {
		SAM::add_mpfr (y, y, a, MPFR_RNDD);
		SAM::add_mpfr (z, z, a, MPFR_RNDU);
	}
	if (_SAM_cancel_tag){
		if (initAccuracy >=  this->nb_significant_digit()+_SAM_cancel_value)
			instability(&_SAM_cancel_count);
	}
	return *this;
}

template <int S>
mp_st<T>& operator+=(const mp_st<S>& a)
{
    int initAccuracy = this->accuracy;
    if (_SAM_cancel_tag && (initAccuracy==DIGIT_NOT_COMPUTED)) {
	initAccuracy = this->nb_significant_digit();
    }
    
    accuracy=DIGIT_NOT_COMPUTED;
    if (SAM_RANDOM) mpfr_add (x, x, a.x, MPFR_RNDU);
    else mpfr_add (x, x, a.x, MPFR_RNDD);
    if (SAM_RANDOM){
	mpfr_add (y, y, a.y, MPFR_RNDU);
	mpfr_add (z, z, a.z, MPFR_RNDD);
    }
    else {
	mpfr_add (y, y, a.y, MPFR_RNDD);
	mpfr_add (z, z, a.z, MPFR_RNDU);
    }  
    
    if (_SAM_cancel_tag){
	if (a.accuracy==DIGIT_NOT_COMPUTED) a.nb_significant_digit();
	if ((initAccuracy < a.accuracy ? initAccuracy : a.accuracy) >= 
	    nb_significant_digit()+_SAM_cancel_value)
	    instability(&_SAM_cancel_count);
    }
    
    return *this;
}

/////////////////////////////////////////////////////////////////////////////////////////////

