/*-----------------------------------------------------------------------
Copyright 2010-2016 S. GRAILLAT, F. JEZEQUEL
Copyright 2010 S. WANG, Y. ZHU
Copyright 2019-2020 F. JEZEQUEL, J.-L. LAMOTTE, J.E. TIDSWELL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
// 
///////////////////////////////////////////////////
// Last Modifications
// 
//
///////////////////////////////////////////////////
// ****m* SAM_eq/operator==
//    NAME
//      operator==
//    SYNOPSIS
//      res = a == b 
//    FUNCTION
//      Define all the functions involving at least one argument
//      of stochastic type which overload the "==" operator.
//      
//      
//    INPUTS
//      a           - an integer, a float, a double, an mpfr_t object or a stochastic number
//      b           - an integer, a float, a double, an mpfr_t object or a stochastic number
//      At least one argument must be of stochastic type.
//    RESULT
//      res         - an integer value
//             
//******
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//   
//    NOTES
//     The return value takes into account the stochastic definition of
//     the "==" operator (see the documentation).
//    BUGS
//   
//    SEE ALSO
//      

template <typename argT>
int operator==(const argT& a) const
{
  mp_st<T>  res;
	
  if (SAM_RANDOM) SAM::sub_mpfr(res.x, x, a, MPFR_RNDU);
  else SAM::sub_mpfr(res.x, x, a, MPFR_RNDD);
	if (SAM_RANDOM){
	    SAM::sub_mpfr(res.y, y, a, MPFR_RNDU);
	    SAM::sub_mpfr(res.z, z, a, MPFR_RNDD);
	}
	else {
	    SAM::sub_mpfr(res.y, y, a, MPFR_RNDD);
	    SAM::sub_mpfr(res.z, z, a, MPFR_RNDU);
	} 
	return (res.numericalnoise())!=0;
}


///////////////////////////////////////////////////////////////////////////////////////////



template <int S>
int operator==(const mp_st<S>& a) const 
{
  mp_st<MP_ST_NBBITS_MAX<S,T>::value> res;

  if (SAM_RANDOM) mpfr_sub (res.x, x, a.x, MPFR_RNDU);
  else mpfr_sub (res.x, x, a.x, MPFR_RNDD);
  if (SAM_RANDOM){
      mpfr_sub (res.y, y, a.y, MPFR_RNDU);
      mpfr_sub (res.z, z, a.z, MPFR_RNDD);
  }
  else {
      mpfr_sub (res.y, y, a.y, MPFR_RNDD);
      mpfr_sub (res.z, z, a.z, MPFR_RNDU);
  }
  
  return res.numericalnoise()!=0;
}












