/*-----------------------------------------------------------------------
Copyright 2023 F. JEZEQUEL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
// 
///////////////////////////////////////////////////
// Last Modification 
//
//
///////////////////////////////////////////////////
//
//contains the fma operator
//  fma(mp_st, mp_st, mp_st)    
//  
//////////////////////////////////////////////////

//template <typename argT>
friend mp_st fma(const mp_st& a, const mp_st& b, const mp_st& c)
{

  mp_st<T> res;
  
  if(_SAM_mul_tag){
      if (a.accuracy==DIGIT_NOT_COMPUTED) a.nb_significant_digit();
      if (b.accuracy==DIGIT_NOT_COMPUTED) b.nb_significant_digit();
      if (a.accuracy==0 && b.accuracy==0) instability(&_SAM_mul_count);
  } 
  res.accuracy=DIGIT_NOT_COMPUTED;
  
  if (SAM_RANDOM) mpfr_fma(res.x, a.x, b.x, c.x, MPFR_RNDU); 
  else mpfr_fma(res.x, a.x, b.x, c.x, MPFR_RNDD);
  if (SAM_RANDOM){
    mpfr_fma(res.y, a.y, b.y, c.y, MPFR_RNDU);
    mpfr_fma(res.z, a.z, b.z, c.z, MPFR_RNDD);
  }
  else{
    mpfr_fma(res.y, a.y, b.y, c.y, MPFR_RNDD);
    mpfr_fma(res.z, a.z, b.z, c.z, MPFR_RNDU);
  }

  if(_SAM_cancel_tag){
    if (a.accuracy==DIGIT_NOT_COMPUTED)  a.nb_significant_digit();	      
    if (b.accuracy==DIGIT_NOT_COMPUTED)  b.nb_significant_digit();
    if (c.accuracy==DIGIT_NOT_COMPUTED)  c.nb_significant_digit();						
    if ((a.accuracy < b.accuracy ? (a.accuracy < c.accuracy ? a.accuracy : c.accuracy):
	 (b.accuracy < c.accuracy ? b.accuracy : c.accuracy)) >=		
	res.nb_significant_digit()+_SAM_cancel_value)			
      _SAM_cancel_count++;
  }
  return res; 
}
/////////////////////////////////////////////////////////////////////////////////////////////
