/*-----------------------------------------------------------------------
Copyright 2010-2016 S. GRAILLAT, F. JEZEQUEL
Copyright 2010 S. WANG, Y. ZHU
Copyright 2019-2020 F. JEZEQUEL, J.-L. LAMOTTE, J.E. TIDSWELL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
//
///////////////////////////////////////////////////
// Last Modifications
// 
//
///////////////////////////////////////////////////
//****m* SAM_mul/operator*
//    NAME
//      operator*
//    SYNOPSIS
//      res = a * b 
//    FUNCTION
//      Defines all the functions involving at least one argument
//      of stochastic type which overload the "*" operator
//      in a statement such as "a*b".
//      
//    INPUTS
//      a           - an integer, a float, a double, an mpfr_t object or a stochastic number
//      b           - an integer, a float, a double, an mpfr_t object or a stochastic number
//      At least one argument must be of stochastic type.
//    RESULT
//      res         - a stochastic number
//             
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//  
//    NOTES
//  
//  
//    BUGS
//  
//  
//    SEE ALSO
//      
//      
//  /


template<typename arg> 
mp_st<T>& operator*=(const arg& a)
{
	accuracy=DIGIT_NOT_COMPUTED;
	
	if (SAM_RANDOM) SAM::mul_mpfr (x, x, a, MPFR_RNDU);
	else SAM::mul_mpfr (x, x, a, MPFR_RNDD);
	if (SAM_RANDOM){
		SAM::mul_mpfr (y, y, a, MPFR_RNDU);
		SAM::mul_mpfr (z, z, a, MPFR_RNDD);
	}
	else {
		SAM::mul_mpfr (y, y, a, MPFR_RNDD);
		SAM::mul_mpfr (z, z, a, MPFR_RNDU);
	}

	return *this;
}


///////////////////////////////////////////////////////////////////////////////////////////

template<int S>
mp_st<T>& operator*=(const mp_st<S>& a)	
{
	if(_SAM_mul_tag) {
		if (accuracy==DIGIT_NOT_COMPUTED) nb_significant_digit();	
		if (accuracy==0){
			if (a.accuracy==DIGIT_NOT_COMPUTED) a.nb_significant_digit();
			if (a.accuracy==0)
				instability(&_SAM_mul_count);
		}
	}
	accuracy=DIGIT_NOT_COMPUTED;
	
	if (SAM_RANDOM) mpfr_mul (x, x, a.x, MPFR_RNDU);
	else mpfr_mul (x, x, a.x, MPFR_RNDD);
	if (SAM_RANDOM){
		mpfr_mul (y, y, a.y, MPFR_RNDU);
		mpfr_mul (z, z, a.z, MPFR_RNDD);
	}
	else {
		mpfr_mul (y, y, a.y, MPFR_RNDD);
		mpfr_mul (z, z, a.z, MPFR_RNDU);
	}
	return *this;
}



///////////////////////////////////////////////////////////////////////////////////////////


