/*-----------------------------------------------------------------------
Copyright 2020 F. JEZEQUEL, J.-L. LAMOTTE

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
//
///////////////////////////////////////////////////
// Last Modifications
// 
//
///////////////////////////////////////////////////

// save the rounding mode to compute the number of significant digits
// always with the same rounding mode



//****m* cadna_numericalnoise/numericalnoise
//    NAME
//      numericalnoise
//    SYNOPSIS
//      res = numericalnoise(x)
//    FUNCTION
//      The numericalnoise() function returns 1 if x is a numerical noise,
//      -1 if x is zero, 0 otherwise
//
//
//    INPUTS
//      x           - a stochastic number
//    RESULT
//      res         - an integer value
//
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//
//
//    NOTES
//
//
//    BUGS
//
//
//    SEE ALSO
//
//
//  /

int numericalnoise() const
{
	int res=1;
	mpfr_t x0, x1, x2, xx;

	mp_rnd_t default_rnd = mpfr_get_default_rounding_mode();
	mpfr_init2 (x0,T);
	mpfr_init2 (x1,T);
	mpfr_init2 (x2,T);
	mpfr_init2 (xx,T);
	mpfr_set_default_rounding_mode(MPFR_RNDN);
	mp_rnd_t tmp_rnd = mpfr_get_default_rounding_mode();
	mpfr_add(xx,x,y,tmp_rnd);
	mpfr_add(xx,xx,z,tmp_rnd);

	if (mpfr_zero_p(xx)){
	    if (mpfr_zero_p(x) && mpfr_zero_p(y)){
		// in this case z==0
		res=-1;  
	    }
	}
	else {
	    if (!mpfr_nan_p(xx)){ 
		    mpfr_d_div(xx, 3.0, xx, tmp_rnd);
		    mpfr_mul(x0, x, xx, tmp_rnd);
		    mpfr_sub_d(x0, x0, 1.0, tmp_rnd);
		    mpfr_mul(x1, y, xx, tmp_rnd);
		    mpfr_sub_d(x1, x1, 1.0, tmp_rnd);
		    mpfr_mul(x2, z, xx, tmp_rnd);
		    mpfr_sub_d(x2, x2, 1.0, tmp_rnd);
		    mpfr_sqr(x0, x0, tmp_rnd);
		    mpfr_sqr(x1, x1, tmp_rnd);
		    mpfr_sqr(x2, x2, tmp_rnd);
		    mpfr_add(xx, x0, x1, tmp_rnd);
		    mpfr_add(xx, xx, x2, tmp_rnd);
		    mpfr_mul_d(xx, xx, 3.0854661704166664, tmp_rnd);
		    res=(mpfr_cmp_d(xx, 0.1)>0);
	    }
	}
	mpfr_set_default_rounding_mode(default_rnd);
		
	if (_SAM_branching_tag && res==1) instability(&_SAM_branching_count);
	
	mpfr_clears(x0, x1, x2, xx, (mpfr_ptr) 0);
	return res; 
} 
