/*-----------------------------------------------------------------------
Copyright 2020 F. JEZEQUEL, J.-L. LAMOTTE

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
//
///////////////////////////////////////////////////
// Last Modifications
// 
//
///////////////////////////////////////////////////


// Condition:
// - the number of bits of mp_st<T> is greater than the u
// for integers values -> no perturbation
// for floating-point values, the last bit is perturbed. 


template <int S>
inline  int  perturbationLastBit(mp_st<T> *mpst, const mp_st<S> & u){
    
    mpfr_t coeff, coefx, coefy, two, exponent;
    int range;

    mpfr_init2 (coeff, (mpfr_prec_t) T);
    mpfr_init2 (coefx, (mpfr_prec_t) T);
    mpfr_init2 (coefy, (mpfr_prec_t) T);
    mpfr_init2 (two, (mpfr_prec_t) T);
    mpfr_init2 (exponent, (mpfr_prec_t) T);
    
    mpfr_set_ui(two ,2, MPFR_RNDN);
    mpfr_set_si(exponent ,-(S+1), MPFR_RNDN);
    mpfr_pow(coeff,two, exponent, MPFR_RNDN);
    
    range=rand()%5+1;
    mpfr_mul_si(coefx,coeff,range,MPFR_RNDN);
    mpfr_mul(coefx,x,coefx, MPFR_RNDU);
    mpfr_add(x,x,coefx,  MPFR_RNDU);
    
    range=rand()%5+1;
    mpfr_mul_si(coefy,coeff,range,MPFR_RNDN);
    mpfr_mul(coefy,y,coefy, MPFR_RNDD);
    mpfr_sub(y,y,coefy, MPFR_RNDD);

    mpfr_clear(coeff);
    mpfr_clear(coefx);
    mpfr_clear(coefy);
    mpfr_clear(two);
    mpfr_clear(exponent);
    
    // perturbation of the last bit doesn't modify the accuracy of the stochastic number
    return T;
}

inline  int  perturbationLastBit(mp_st<T> *mpst, const mpfr_t &  u){
    
    mpfr_t coeff, coefx, coefy, two, exponent;
    int range;
    
    mpfr_init2 (coeff, (mpfr_prec_t) T);
    mpfr_init2 (coefx, (mpfr_prec_t) T);
    mpfr_init2 (coefy, (mpfr_prec_t) T);
    mpfr_init2 (two, (mpfr_prec_t) T);
    mpfr_init2 (exponent, (mpfr_prec_t) T);
    
    mpfr_set_ui(two ,2, MPFR_RNDN);
    mpfr_set_si(exponent ,-(SAM::mp_st_nbbit<mpfr_t>::prec(u)+1), MPFR_RNDN);
    mpfr_pow(coeff,two, exponent, MPFR_RNDN);
    
    range=rand()%5+1;
    mpfr_mul_si(coefx,coeff,range,MPFR_RNDN);
    mpfr_mul(coefx,mpst->x,coefx,  MPFR_RNDU);
    mpfr_add(mpst->x,mpst->x,coefx,  MPFR_RNDU);
    
    range=rand()%5+1;
    mpfr_mul_si(coefy,coeff,range,MPFR_RNDN);
    mpfr_mul(coefy,mpst->y,coefy,  MPFR_RNDD);
    mpfr_sub(mpst->y,mpst->y,coefy,  MPFR_RNDD);

    mpfr_clear(coeff);
    mpfr_clear(coefx);
    mpfr_clear(coefy);
    mpfr_clear(two);
    mpfr_clear(exponent);
    
    return SAM::mp_st_nbbit<mpfr_t>::prec(u);
}


inline  int perturbationLastBit(mp_st<T> *mpst, const double &  u){
    
    mpfr_t coeff, coefx, coefy, two, exponent;
    int range;
    
    mpfr_init2 (coeff, (mpfr_prec_t) T);
    mpfr_init2 (coefx, (mpfr_prec_t) T);
    mpfr_init2 (coefy, (mpfr_prec_t) T);
    mpfr_init2 (two, (mpfr_prec_t) T);
    mpfr_init2 (exponent, (mpfr_prec_t) T);
    
    mpfr_set_ui(two ,2, MPFR_RNDN);
    mpfr_set_si(exponent ,-(SAM::mp_st_nbbit<double>::prec(u)+1), MPFR_RNDN);
    mpfr_pow(coeff,two, exponent, MPFR_RNDN);
    
    range=rand()%5+1;
    mpfr_mul_si(coefx,coeff,range,MPFR_RNDN);
    mpfr_mul(coefx,mpst->x,coefx,  MPFR_RNDU);
    mpfr_add(mpst->x,mpst->x,coefx,  MPFR_RNDU);
    
    range=rand()%5+1;
    mpfr_mul_si(coefy,coeff,range,MPFR_RNDN);
    mpfr_mul(coefy,mpst->y,coefy,  MPFR_RNDD);
    mpfr_sub(mpst->y,mpst->y,coefy,  MPFR_RNDD);
    
    mpfr_clear(coeff);
    mpfr_clear(coefx);
    mpfr_clear(coefy);
    mpfr_clear(two);
    mpfr_clear(exponent);
    
    return SAM::mp_st_nbbit<double>::prec(u);
}
 
inline  int perturbationLastBit(mp_st<T> *mpst, const float  &  u){
    
    mpfr_t coeff, coefx, coefy, two, exponent;
    int range;
    
    mpfr_init2 (coeff, (mpfr_prec_t) T);
    mpfr_init2 (coefx, (mpfr_prec_t) T);
    mpfr_init2 (coefy, (mpfr_prec_t) T);
    mpfr_init2 (two, (mpfr_prec_t) T);
    mpfr_init2 (exponent, (mpfr_prec_t) T);
    
    mpfr_set_ui(two ,2, MPFR_RNDN);
    mpfr_set_si(exponent ,-(SAM::mp_st_nbbit<float>::prec(u)+1), MPFR_RNDN);
    mpfr_pow(coeff,two, exponent, MPFR_RNDN);
    
    range=rand()%5+1;
    mpfr_mul_si(coefx,coeff,range,MPFR_RNDN);
    mpfr_mul(coefx,mpst->x,coefx,  MPFR_RNDU);
    mpfr_add(mpst->x,mpst->x,coefx,  MPFR_RNDU);

    range=rand()%5+1;
    mpfr_mul_si(coefy,coeff,range,MPFR_RNDN);
    mpfr_mul(coefy,mpst->y,coefy,  MPFR_RNDD);
    mpfr_sub(mpst->y,mpst->y,coefy,  MPFR_RNDD);

    mpfr_clear(coeff);
    mpfr_clear(coefx);
    mpfr_clear(coefy);
    mpfr_clear(two);
    mpfr_clear(exponent);

    return SAM::mp_st_nbbit<float>::prec(u);
}

inline int perturbationLastBit(mp_st<T> *mpst, const int  &u){ return T; }
inline int perturbationLastBit(mp_st<T> *mpst, const long  &u){ return T; }  
inline int perturbationLastBit(mp_st<T> *mpst, const unsigned int  &u){ return T; } 
inline int perturbationLastBit(mp_st<T> *mpst, const unsigned long  &u){ return T; } 
