/*-----------------------------------------------------------------------
Copyright 2010-2016 S. GRAILLAT, F. JEZEQUEL
Copyright 2010 S. WANG, Y. ZHU
Copyright 2019-2020 F. JEZEQUEL, J.-L. LAMOTTE, J.E. TIDSWELL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
// 
///////////////////////////////////////////////////
// Last Modification 
//
//
///////////////////////////////////////////////////



//////////////////////////////////////////////////////////



//****f* SAM_str/strp
//    NAME
//     strp
//
//    SYNOPSIS
//      char* strp(mp_st&)
//    FUNCTION
//      The output string contains the scientific notation of the 
//      stochastic argument; only the exact significant digits appear 
//      in the string. The strp function must be used only with
//      the family of printf functions. The only restriction is that
//      it is not possible to have more than 256 calls to the strp
//      function in one call to the printf function.
//    INPUTS
//      The strp function has a stochastic argument.  
//    RESULT
//      It returns a string.
//    SEE ALSO
//      str(3)
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//  
//    NOTES
//  
//  
//    BUGS
//  
//  
//      
//      
//  /


template <int T>
char* strp(const mp_st<T>& a)
{
	char *s;
	
	s=SAM_chstr[SAM_numstr];
	if ((++SAM_numstr) == __SAM_MAXNBSTRING )  SAM_numstr=0; 
	return a.str(s);
}




template <int  T>
std::ostream& operator <<(std::ostream& s, const mp_st<T> &a) 
{
    int CL = 10 +  a.getPrecision();
    
    if (CL >= __SAM_MAXSIZESTRING) {
	cerr << " MPFR number are too large for SAM internal strings." << endl;
	exit(1);
    }
    char *ch;
    
    ch=SAM_chstr[SAM_numstr];
    if ((++SAM_numstr) == __SAM_MAXNBSTRING )  SAM_numstr=0; 
    
    return s << a.str(ch);
};

template <int T>
istream& operator >>(istream& s, mp_st<T>& a)
 {
 	double d;
 	s>>d;

	a=mp_st<T>(d);
 	return s;
 }
